package com.zhentao.util;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class SignUtil {

    /**
     * 生成签名
     * @param params 请求参数
     * @param secret 密钥
     * @return 签名
     */
    public static String generateSign(Map<String, String> params, String secret) {
        // 1. 按ASCII码排序参数
        List<String> keys = new ArrayList<>(params.keySet());
        Collections.sort(keys);

        // 2. 拼接key=value&字符串
        StringBuilder paramStr = new StringBuilder();
        for (String key : keys) {
            paramStr.append(key).append("=").append(params.get(key)).append("&");
        }

        // 3. 拼接secret
        paramStr.append("key=").append(secret);

        // 4. MD5加密
        return md5(paramStr.toString());
    }

    /**
     * 验证签名
     * @param params 请求参数
     * @param sign 签名
     * @param secret 密钥
     * @return 是否验证通过
     */
    public static boolean verifySign(Map<String, String> params, String sign, String secret) {
        String generatedSign = generateSign(params, secret);
        return generatedSign.equals(sign);
    }

    /**
     * MD5加密
     * @param input 输入字符串
     * @return 加密后的字符串
     */
    private static String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] digest = md.digest(input.getBytes());
            StringBuilder sb = new StringBuilder();
            for (byte b : digest) {
                sb.append(String.format("%02x", b & 0xff));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("MD5算法不可用", e);
        }
    }
}
